@@ -6,7 +6,7 @@ use warnings;
 
 BEGIN {
 	$Types::Standard::Tuple::AUTHORITY = 'cpan:TOBYINK';
-	$Types::Standard::Tuple::VERSION   = '1.010001';
+	$Types::Standard::Tuple::VERSION   = '1.012000';
 }
 
 $Types::Standard::Tuple::VERSION =~ tr/_//d;
@@ -28,14 +28,14 @@ sub __constraint_generator
 	if (exists $constraints[-1] and ref $constraints[-1] eq "HASH")
 	{
 		$slurpy = Types::TypeTiny::to_TypeTiny($constraints[-1]{slurpy});
-		Types::TypeTiny::TypeTiny->check($slurpy)
+		Types::TypeTiny::is_TypeTiny($slurpy)
 			or _croak("Slurpy parameter to Tuple[...] expected to be a type constraint; got $slurpy");
 		pop(@constraints)->{slurpy} = $slurpy;  # keep canonicalized version around
 	}
 	
 	for (@constraints)
 	{
-		Types::TypeTiny::TypeTiny->check($_)
+		Types::TypeTiny::is_TypeTiny($_)
 			or _croak("Parameters to Tuple[...] expected to be type constraints; got $_");
 	}
 	
